#!/usr/bin/env python
# Copyright 2012  Andrey Tataranovich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import fcntl
import dbus
import gobject
import time
from dbus.mainloop.glib import DBusGMainLoop

# Fork to background
if os.fork():
	sys.exit(0)

xkbmap = os.path.expanduser('~/.Xkbmap')
if not os.path.isfile(xkbmap):
	print('File not found: ' + xkbmap)
	sys.exit(1)

# Try to detect if another instance already running
lock = open(os.path.realpath(xkbmap), 'r')
try:
	fcntl.flock(lock, fcntl.LOCK_EX|fcntl.LOCK_NB)
except:
	print('Unable to lock file: ' + xkbmap + '. Maybe another instance already running')
	sys.exit(1)

dbus_loop = DBusGMainLoop()
bus = dbus.SystemBus(mainloop=dbus_loop)

# Setup DBus handler to execute 'setxkbmap $(cat ~/.Xkbmap)' after
# resume from sleep
command = 'setxkbmap $(cat ' + xkbmap + ')'
def handler(sender=None):
	if os.path.isfile(xkbmap):
		time.sleep(5)
		os.system(command)

bus.add_signal_receiver(handler, signal_name='Resuming', dbus_interface='org.freedesktop.UPower', path='/org/freedesktop/UPower')
bus.add_signal_receiver(handler, signal_name='ActiveSessionChanged', dbus_interface='org.freedesktop.ConsoleKit.Seat')

loop = gobject.MainLoop()
loop.run()
